# -*- coding: utf-8 -*-

import os
import sys
import argparse
from signal import signal, SIGPIPE, SIG_DFL

import logging
from logging.handlers import SysLogHandler
import time

import Pyro4
from service import find_syslog, Service

from . import yawap_tools

signal(SIGPIPE, SIG_DFL)

PYRO_OBJ_ID = "yawap"
UDS_YAWAP = "/tmp/yawap.s"

WIFI_NETWORK_LIST_FOLDER = "/var/lib/yawap/"
WIFI_NETWORK_LIST_FILE = WIFI_NETWORK_LIST_FOLDER + "scanned_networks"


class YawapService(Service):
    def __init__(self, *args, **kwargs):
        super(YawapService, self).__init__(*args, **kwargs)
        self.logger.addHandler(
            SysLogHandler(
                address=find_syslog(), facility=SysLogHandler.LOG_DAEMON
            )
        )
        self.logger.setLevel(logging.INFO)

    def run(self):
        self.logger.info("Start Daemon")
        if os.path.exists(UDS_YAWAP):
            os.unlink(UDS_YAWAP)
        daemon = Pyro4.Daemon(unixsocket=UDS_YAWAP)

        yawap_instance = yawap_tools.Yawap(self.logger)
        daemon.register(yawap_instance, objectId=PYRO_OBJ_ID)

        daemon.requestLoop(loopCondition=lambda: not self.got_sigterm())

        self.logger.info("Stop Daemon")


def parse():
    parser = argparse.ArgumentParser(
        description="Check internet connectivity and manage access point. "
        "If started without arguments, "
        "and a internet connection is not available, "
        "then it scans wifi networks and starts in AP mode"
        "It is possible to manually turn on or off the AP. "
        "And it is possible to add Wifi network or get the list "
        "of available network found."
    )
    parser.add_argument("--service", nargs=1)
    parser.add_argument(
        "--install",
        nargs=4,
        help="Install hostapd, dnsmasq. "
        "Usage: --install INTERFACE SSID PASSWD COUNTRYCODE",
    )
    parser.add_argument(
        "--on", action="store_true", help="Start the access point"
    )
    parser.add_argument(
        "--off",
        action="store_true",
        help="Stop the access point " "and try to connect to Wifi",
    )
    parser.add_argument(
        "--list", action="store_true", help="Return the scanned Wifi Networks"
    )
    parser.add_argument(
        "--list-saved",
        action="store_true",
        help="Return the Saved Wifi Networks",
    )
    parser.add_argument(
        "--add",
        nargs=2,
        help="Connect to the network given." "Usage: --add SSID Key",
    ),
    parser.add_argument(
        "--network-conf",
        nargs=2,
        action="append",
        help="Extra network configuration for wpa_supplicant."
        "Usage: --network-conf KEY VALUE"
        "KEY and VALUE are valid wpa_supplicant params",
    ),
    parser.add_argument(
        "--delete",
        nargs=1,
        help="Delete the network given." "Usage: --del SSID",
    )

    return parser.parse_args()


def main():
    args = parse()

    service = YawapService("yawap", pid_dir="/tmp")

    if args.service is not None:
        if args.service[0] == "start" and not service.is_running():
            logging.info("starting...")
            service.start()

            # Waiting for the service to start
            # and create the communication channel
            t = time.time()
            while (time.time() - t) < 20:
                if service.is_running():
                    break
                time.sleep(0.2)
            else:
                logging.error("Can't start the service")
                service.kill()
                sys.exit(-1)

            t = time.time()
            while (time.time() - t) < 5:
                if os.path.exists(UDS_YAWAP):
                    break
                time.sleep(0.2)
            else:
                logging.error("Can't communicate with the service")
                service.kill()
                sys.exit(-1)

            yawap_make = Pyro4.Proxy(
                "PYRO:" + PYRO_OBJ_ID + "@./u:" + UDS_YAWAP
            )

            yawap_make.turn_off_ap()
            if not yawap_make.is_connected_to_internet():
                logging.info("Not connected to internet, started the AP")

                networks = yawap_make.scan_networks()

                with open(WIFI_NETWORK_LIST_FILE, "w") as fp:
                    fp.write(";".join(networks))

                yawap_make.turn_on_ap()

            else:
                logging.info("Connected :) ! Leaving")

        elif args.service[0] == "stop" and service.is_running():
            logging.info("stoping...")
            service.stop()
        elif args.service[0] == "status":
            if service.is_running():
                logging.info("Service is running.")
            else:
                logging.info("Service is not running.")

    else:
        yawap_make = Pyro4.Proxy("PYRO:" + PYRO_OBJ_ID + "@./u:" + UDS_YAWAP)

        if args.install is not None:
            yawap_instance = yawap_tools.Yawap(
                logging.getLogger("Yawap Installer")
            )
            yawap_instance.install(
                args.install[1],
                args.install[2],
                interface=args.install[0],
                iso_country_code=args.install[3],
            )

        elif args.on:
            yawap_make.turn_on_ap()

        elif args.off:
            yawap_make.turn_off_ap()

        elif args.list:
            # if ap mode started
            with open(WIFI_NETWORK_LIST_FILE) as f:
                print(f.read())

        elif args.list_saved:
            networks = yawap_make.list_saved()
            print(";".join(networks))

        elif args.add is not None:
            logging.info(f"Adding network: {args.add[0]}")
            extra_conf = (
                {el[0]: el[1] for el in args.network_conf}
                if args.network_conf is not None
                else {}
            )
            yawap_make.add_network(args.add[0], args.add[1], **extra_conf)
            yawap_make.turn_off_ap()

        elif args.delete is not None:
            logging.info(f"Deleting network: {args.delete[0]}")
            yawap_make.del_network(args.delete[0])



# -*- coding: utf-8 -*-

import yawap.wpasupplicantconf as wsc

import os
import shutil
import time
import subprocess
from signal import signal, SIGPIPE, SIG_DFL
import Pyro4

from yawap import iso_country_codes as icc

signal(SIGPIPE, SIG_DFL)

WIFI_NETWORK_LIST_FOLDER = "/var/lib/yawap/"
WIFI_NETWORK_LIST_FILE = WIFI_NETWORK_LIST_FOLDER + "scanned_networks"

WPA_SUPPLICANT_FILE = "/etc/wpa_supplicant/wpa_supplicant.conf"


@Pyro4.expose
class Yawap(object):
    def __init__(self, logger):
        self.logger = logger

    def popen(self, cmd):
        self.logger.info(f'RUN > "{" ".join(cmd)}"')
        ret = subprocess.run(cmd, capture_output=True)
        return (ret.returncode, ret.stdout)

    def install(
        self, ap_name, ap_passwd, interface="wlan0", iso_country_code="EN"
    ):
        self.popen(["systemctl", "unmask", "hostapd"])

        self.popen(["systemctl", "disable", "dnsmasq"])
        self.popen(["systemctl", "disable", "hostapd"])
        time.sleep(2)

        self.popen(["systemctl", "stop", "dnsmasq"])
        self.popen(["systemctl", "stop", "hostapd"])
        time.sleep(2)

        # dhcpcd
        if not os.path.isfile("/etc/dhcpcd.conf.source"):
            with open("/etc/dhcpcd.conf", "r") as fp_dhcpcd:
                dhcpcd = fp_dhcpcd.read()

            dhcpcd += "\n" "noarp\n" "timeout 2\n" "retry 5\n"
            with open("/etc/dhcpcd.conf.source", "w") as fp_dhcpcd_source:
                fp_dhcpcd_source.write(dhcpcd)

        # wpa_supplicant
        if iso_country_code in icc.CC:
            wpa = wsc.WpaSupplicantConf(WPA_SUPPLICANT_FILE)
            wpa.fields()["country"] = iso_country_code

            wpa.write(WPA_SUPPLICANT_FILE)

        # dnsmasq
        if not os.path.isfile("/etc/dnsmasq.conf.orig"):
            shutil.copy("/etc/dnsmasq.conf", "/etc/dnsmasq.conf.orig")

        with open("/etc/dnsmasq.conf", "w") as dnsmasq:
            dnsmasq.write("interface={}\n".format(interface))
            dnsmasq.write(
                "dhcp-range=192.168.4.2,192.168.4.20,255.255.255.0,24h\n"
            )

        # self.popen(["systemctl reload dnsmasq')

        # "echo 'nickw444  ALL=(ALL:ALL) ALL' >> /etc/sudoers"

        # hostapd
        with open("/etc/hostapd/hostapd.conf", "w") as hostapd:
            hostapd.write(
                """\
interface={}
driver=nl80211
ssid={}
hw_mode=g
channel=7
wmm_enabled=0
macaddr_acl=0
auth_algs=1
ignore_broadcast_ssid=0
wpa=2
wpa_passphrase={}
wpa_key_mgmt=WPA-PSK
wpa_pairwise=TKIP
rsn_pairwise=CCMP""".format(
                    interface, ap_name, ap_passwd
                )
            )

        with open("/etc/default/hostapd", "r") as hostapd_default:
            had = hostapd_default.read()

        had = had.replace(
            '#DAEMON_CONF=""', 'DAEMON_CONF="/etc/hostapd/hostapd.conf"'
        )
        with open("/etc/default/hostapd", "w") as hostapd_default:
            hostapd_default.write(had)

        # ssid list dir
        if not os.path.isdir(WIFI_NETWORK_LIST_FOLDER):
            os.makedirs(WIFI_NETWORK_LIST_FOLDER)

        # service
        service_file_data = """\
[Unit]
Description=Yet Another Wifi Access Point Daemon
After=network-online.target

[Service]
Type=forking
ExecStart=/usr/local/bin/yawap --service start
ExecStop=/usr/local/bin/yawap --service stop

[Install]
WantedBy=multi-user.target

    """

        with open("/etc/systemd/system/yawap.service", "w") as service_file:
            service_file.write(service_file_data)

        self.popen(["systemctl", "enable", "yawap"])

    def scan_networks(self, interface="wlan0"):
        self.logger.info("Scanning available networks")
        cmd = """iwlist {} scan | grep -ioE 'SSID:"(.*)"'""".format(interface)
        result = os.popen(cmd)  # keep it easy for now
        result = list(result)
        ssid_list = []

        if "Device or resource busy" not in result:
            ssid_list = [
                item.lstrip("SSID:").strip("\n").strip('"') for item in result
            ]

        return [i for i in set(ssid_list) if i != ""]

    def turn_on_ap(self):
        self.logger.info("Starting AP")
        self.popen(["systemctl", "stop", "dhcpcd"])

        with open("/etc/dhcpcd.conf.source") as dhcpcd_fd:
            dhcpcd_src = dhcpcd_fd.read()

        with open("/etc/dhcpcd.conf", "w") as dhcpcd_fd:
            dhcpcd_fd.write(dhcpcd_src)
            lines = (
                "interface wlan0\n"
                "    static ip_address=192.168.4.1/24\n"
                "    nohook wpa_supplicant\n"
            )
            dhcpcd_fd.write(lines)

        # self.popen(["systemctl daemon-reload")
        self.popen(["systemctl", "start", "dhcpcd"])

        self.popen(["systemctl", "start", "dnsmasq"])
        self.popen(["systemctl", "start", "hostapd"])

    def turn_off_ap(self):
        self.logger.info("[Stoping AP]")
        self.popen(["systemctl", "stop", "hostapd"])
        self.popen(["systemctl", "stop", "dnsmasq"])

        self.popen(["ip", "addr", "flush", "dev", "wlan0"])
        self.popen(["ip", "link", "set", "dev", "wlan0", "up"])

        self.popen(["systemctl", "stop", "dhcpcd"])
        self.popen(["cp", "/etc/dhcpcd.conf.source", "/etc/dhcpcd.conf"])
        # self.popen(["systemctl daemon-reload"])

        self.popen(["systemctl", "start", "dhcpcd"])

    def is_connected_to_internet(self):
        cmd = "ping -q -w 1 -c 1 8.8.8.8 > /dev/null && echo ok || echo error"

        self.logger.info("Checking Internet Connection")
        t = time.time()
        connected = False
        while time.time() - t < 10:
            # ping google gateway
            process = subprocess.Popen(
                cmd, stdout=subprocess.PIPE, stderr=None, shell=True
            )
            output, _ = process.communicate()

            connected = output.find(b"ok") != -1
            if connected:
                break

        return connected

    def add_network(self, ssid, passwd, **extra_conf):
        if "key_mgmt" not in extra_conf:
            extra_conf["key_mgmt"] = "WPA-PSK"

        conf = wsc.WpaSupplicantConf(WPA_SUPPLICANT_FILE)

        conf.add_network(ssid, psk='"{}"'.format(passwd), **extra_conf)

        conf.write(WPA_SUPPLICANT_FILE)

    def del_network(self, ssid):
        conf = wsc.WpaSupplicantConf(WPA_SUPPLICANT_FILE)
        conf.remove_network(ssid)
        conf.write(WPA_SUPPLICANT_FILE)

    def list_saved(self):
        try:
            conf = wsc.WpaSupplicantConf(WPA_SUPPLICANT_FILE)
            networks = conf.networks().keys()
        except (wsc.ParseError, IOError) as e:
            self.logger.error(str(e))
            networks = []

        return networks



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages
from setuptools import Distribution
from setuptools.command.install import install
from yawap import __version__

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = ["Pyro4", "service"]

setup_requirements = []

test_requirements = []


setup(
    author="Vivien Henry",
    author_email="vivien.henry@inductivebrain.fr",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    description="Access point and wifi connection tool",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="yawap",
    name="yawap",
    packages=find_packages(include=["yawap"]),
    scripts=["bin/yawap"],
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/lukh/yawap",
    version=__version__,
    zip_safe=False,
)
